/*
 * uart.c
 *
 * Created: 21-07-2023 17:12:59
 *  Author: controllerstech
 */ 

#include <xc.h>
#include <avr/interrupt.h>
#include "uart.h"

void timer0_init (void)
{
	TCCR0A = 1<<WGM01;  // enable CTC for TIM0
	OCR0A = 103;  // 1000000/104 = 9615 baud rate
	TIMSK = 1<<OCIE0A;  // enable interrupt for comapare match A
	sei();  // enable interrupt
}

volatile uint16_t maindata = 0;

void uart_char (char c)
{
	maindata = c<<1;  // add the start bit(0) to 1st position
	maindata |= 1<<9;  // add the stop bit(1) to 10th position
	TCCR0B = 1<<CS00;  // start timer no prescaler -> TIMclk = CLKio = 1MHz
}

void uart_str (char *str)
{
	while (*str) // while there is some charcater in tthe string
	{
		uart_char(*str++);  // send the character and increment the pointer
		while (maindata != 0);  // wait until the data is sent
	}
}

void uart_init (void)
{
	timer0_init();
	DDRB = 0x01;  // PB0 as output
	PORTB = 1<<PB0;  // Set PB0 HIGH
}

ISR (TIM0_COMPA_vect)
{
	if (maindata & 0x01) PORTB |= 1<<PB0;  // set the pin PB0
	else PORTB &= ~(1<<PB0);  // Reset the pin PB0
	maindata = maindata >>1;  // shift the data to the right
	
	if (maindata == 0)  // if all the data is sent
	{
		TCCR0B = 0;  // stop the timer
		TCNT0 = 0;  // reset the counter
	}
}